/*******************************************************************************
 *
 * display.c - The main display engine.
 *
 * Creates the display view and provides functions for drawing into it.
 *
 * Cheetah Web Browser
 * Copyright (C) 2001 Garett Spencley 
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. 
 *
 *******************************************************************************/

#include <stdio.h>
#include <string.h>
#include <gdk-pixbuf/gdk-pixbuf.h>
#include <stdlib.h>

#include "display.h"
#include "gui.h"
#include "colors.h"
#include "error.h"

static GtkWidget *display     = NULL;
static GdkPixmap *background  = NULL;

/*
 * configure_event()
 */

static gint configure_event(GtkWidget *widget, GdkEventConfigure *event)
{
	if(background)
		gdk_pixmap_unref(background);

	background = gdk_pixmap_new(widget->window, widget->allocation.width, 
					widget->allocation.height, -1);
	
	gdk_draw_rectangle(background, widget->style->white_gc, TRUE, 0, 0, 
					widget->allocation.width, widget->allocation.height);

	return TRUE;
}

/*
 * expose_event()
 *
 */

static gint expose_event(GtkWidget *widget, GdkEventExpose *event)
{
	gdk_draw_pixmap(widget->window, widget->style->fg_gc[GTK_WIDGET_STATE(widget)], 
				background, event->area.x, event->area.y, event->area.x, event->area.y, 
				event->area.width, event->area.height); 

	return FALSE;
}

static gint button_press_event(GtkWidget *widget, GdkEventButton *event)
{
  	return TRUE;
}

static gint motion_notify_event(GtkWidget *widget, GdkEventMotion *event)
{
  	return TRUE;
}

/* 
 * init_display_area()
 *
 * initialize the display area 
 */

int init_display_area(GtkWidget *box)
{
	if(!box) 
		return -1;

	display = gtk_drawing_area_new();
	if(!display)
		return -1;

	gtk_drawing_area_size(GTK_DRAWING_AREA(display), 800, 600);
	gtk_box_pack_start(GTK_BOX(box), display, TRUE, TRUE, 0);
	gtk_widget_show(display);

	gtk_signal_connect(GTK_OBJECT(display), "expose_event", (GtkSignalFunc) expose_event, NULL);
	gtk_signal_connect(GTK_OBJECT(display), "configure_event", (GtkSignalFunc) configure_event, NULL);

	gtk_signal_connect (GTK_OBJECT (display), "motion_notify_event", (GtkSignalFunc) motion_notify_event, NULL);
	gtk_signal_connect (GTK_OBJECT (display), "button_press_event", (GtkSignalFunc) button_press_event, NULL);

  	gtk_widget_set_events (display, GDK_EXPOSURE_MASK
			 | GDK_LEAVE_NOTIFY_MASK
			 | GDK_BUTTON_PRESS_MASK
			 | GDK_POINTER_MOTION_MASK
			 | GDK_POINTER_MOTION_HINT_MASK);

	return 0;
}

/* FIXME: There are easiser (and probably faster) ways to do this
 * learn them soon */

void set_rgb_color(GdkGC *gc, GdkColor *color)
{
	char *rgbbuf, *pos;
	int x, y;

	rgbbuf = (char *)malloc(sizeof(char) * display->allocation.width * 
							display->allocation.height * 3);

	pos = rgbbuf;

	for(y = 0; y < display->allocation.height; y++) {
		for(x = 0; x < display->allocation.width; x++) {
			*pos++ = color->red;
			*pos++ = color->green;
			*pos++ = color->blue;
		}
	}

	gdk_draw_rgb_image(display->window, gc,
		      0, 0, display->allocation.width, display->allocation.height,
		      GDK_RGB_DITHER_MAX, rgbbuf, display->allocation.width * 3);

	free(rgbbuf);
}

int display_set_bgcolor(char *rgb) 
{
	char *hex;
	GdkColor *color;
	GdkGC    *gc;

	if(*rgb != '#') {

		hex = get_hex_value(rgb);
		if(!hex) 
			return -1;
		
	} else {

		hex = strdup(++rgb);
		if(!hex) 
			return -1;
	}

	color = create_gdk_color(hex);
	
	gc = gdk_gc_new(display->window);
	set_rgb_color(gc, color);

	/* draw the background */

	if(background)
		gdk_pixmap_unref(background);

	background = gdk_pixmap_new(display->window, display->allocation.width, 
					display->allocation.height, -1);

	gdk_draw_rectangle(background, gc, TRUE, 0, 0, 
					display->allocation.width, display->allocation.height);

	free(color);
	free(hex);

	return 0;
}
